import hashlib
from typing import Mapping

from fastapi_rss.models import RSSFeed

from starlette.responses import Response


class RSSResponse(Response):
    '''
    A subclass of starlette.responses.Response which will set the content
    to an RSS XML document. It takes one argument, an RSSFeed object which will
    be converted to an XML document.
    '''
    media_type = 'application/xml'
    charset = 'utf-8'

    @property
    def etag(self):
        '''
        Generates a SHA1 sum of the body of the response so that the server can
        support the ETag protocol.

        :return: The SHA1 hex digest of the body of the response
        :rtype: str
        '''
        return hashlib.sha1(self.body).hexdigest()

    def init_headers(self, headers: Mapping[str, str] = None) -> None:
        newheaders = {
            'Accept-Range': 'bytes',
            'Connection': 'Keep-Alive',
            'ETag': self.etag,
            'Keep-Alive': 'timeout=5, max=100',
        }

        headers = headers or {}
        for headername in newheaders:
            if headername not in headers:
                headers[headername] = newheaders[headername]
        super().init_headers(headers)

    def render(self, rss: RSSFeed) -> bytes:
        return rss.tostring()



import datetime
import getpass
import locale
import os
import subprocess


def git_config(key: str, _global=True):
    cmd = ['git', 'config']
    if _global:
        cmd.append('--global')
    cmd.append(key)
    return subprocess.check_output(cmd).decode('utf8').strip('\n')


def get_locale_code():
    return locale.getdefaultlocale()[0].lower()


def to_camelcase(string):
    string = string.split('_')
    for i, el in enumerate(string):
        if i == 0:
            continue
        string[i] = el.capitalize()
    return ''.join(string)



# flake8: noqa

from fastapi_rss.models.category import Category, CategoryAttrs
from fastapi_rss.models.image import Image
from fastapi_rss.models.cloud import Cloud, CloudAttrs
from fastapi_rss.models.item import Item
from fastapi_rss.models.textinput import TextInput
from fastapi_rss.models.enclosure import Enclosure, EnclosureAttrs
from fastapi_rss.models.guid import GUID, GUIDAttrs
from fastapi_rss.models.source import Source, SourceAttrs

from fastapi_rss.models.feed import RSSFeed



import datetime
from typing import Optional

from pydantic import BaseModel

from fastapi_rss.models.category import Category
from fastapi_rss.models.enclosure import Enclosure
from fastapi_rss.models.guid import GUID
from fastapi_rss.models.source import Source


class Item(BaseModel):
    title: str
    link: Optional[str]
    description: Optional[str]
    author: Optional[str]
    category: Optional[Category]
    comments: Optional[str]
    enclosure: Optional[Enclosure]
    guid: Optional[GUID]
    pub_date: Optional[datetime.datetime]
    source: Optional[Source]



from typing import Optional

from pydantic import BaseModel


class Image(BaseModel):
    url: str
    title: str
    link: str

    width: Optional[int]
    height: Optional[int]
    description: Optional[str]



from typing import Optional

from pydantic import BaseModel, Field


class CategoryAttrs(BaseModel):
    domain: Optional[str]
    '''
    A string that identifies a categorization taxonomy.
    It's a forward-slash separated string which identifies
    a hierarchic location in the indicated taxonomy.

    Processors may establish conventions for the interpretation of categories
    '''


class Category(BaseModel):
    '''
    An optional sub-element of a channel or an item
    '''
    content: str
    attrs: Optional[CategoryAttrs]



from typing import Optional

from pydantic import BaseModel


class CloudAttrs(BaseModel):
    domain: Optional[str]
    port: Optional[str]
    path: Optional[str]
    register_procedure: Optional[str]
    protocol: Optional[str]


class Cloud(BaseModel):
    attrs: Optional[CloudAttrs]



from setuptools import setup

from fastapi_rss import __version__


with open('requirements.txt', 'r') as f:
    requirements = [line for line in f]

with open('readme.md', 'r') as f:
    readme = f.read()


setup(
    name='fastapi_rss',
    packages=['fastapi_rss', 'fastapi_rss.models'],
    license='MIT',
    version=__version__,
    author='Dogeek',
    url='https://github.com/Dogeek/fastapi_rss/',
    description='A library to generate RSS feeds for FastAPI',
    long_description=readme,
    long_description_content_type='text/markdown',
    python_requires='>=3.6.5',
    install_requires=requirements,
    zip_safe=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    platforms='any',
)



from typing import Optional

from pydantic import BaseModel


class EnclosureAttrs(BaseModel):
    url: str
    length: int
    type: str


class Enclosure(BaseModel):
    content: str
    attrs: Optional[EnclosureAttrs]



from datetime import datetime
from typing import Optional, List

from fastapi import __version__ as faversion
from lxml import etree
from pydantic import BaseModel

from fastapi_rss import __version__ as farssversion
from fastapi_rss.utils import get_locale_code, to_camelcase
from fastapi_rss.models.category import Category
from fastapi_rss.models.cloud import Cloud
from fastapi_rss.models.image import Image
from fastapi_rss.models.textinput import TextInput
from fastapi_rss.models.item import Item


class RSSFeed(BaseModel):
    title: str
    link: str
    description: str

    language: str = get_locale_code()
    copyright: Optional[str]
    managing_editor: Optional[str]
    webmaster: Optional[str]
    pub_date: Optional[datetime]
    last_build_date: Optional[datetime]
    category: Optional[List[Category]]
    generator: str = f'FastAPI v{faversion} w/ FastAPI_RSS v{farssversion}'
    docs: str = 'https://validator.w3.org/feed/docs/rss2.html'
    cloud: Optional[Cloud]
    ttl: int = 60
    image: Optional[Image]
    text_input: Optional[TextInput]
    skip_hours: List[int] = []
    skip_days: List[str] = []

    item: List[Item] = []

    @staticmethod
    def generate_tree(root, dict_):  # TODO: improve that shiete
        for key, value in dict_.items():
            if value is None:
                continue
            if isinstance(value, list):
                for item in value:
                    attrs = item.pop('attrs', {})
                    content = item.pop('content', None)
                    itemroot = etree.SubElement(root, to_camelcase(key), attrs)
                    if content is not None:
                        itemroot.text = content
                    else:
                        RSSFeed.generate_tree(itemroot, item)
                continue

            if isinstance(value, BaseModel):
                if hasattr(value, 'attrs'):
                    attrs = value.attrs.dict()
                else:
                    attrs = {}

                if hasattr(value, 'content'):
                    content = value.content
                else:
                    content = None

                element = etree.SubElement(root, to_camelcase(key), attrs)
                if content:
                    element.text = content
                continue

            if isinstance(value, datetime):
                value = value.strftime('%a, %d %b %Y %H:%M:%S GMT')
            element = etree.SubElement(root, to_camelcase(key))
            element.text = str(value)

    def tostring(self):
        rss = etree.Element('rss', version='2.0')
        channel = etree.SubElement(rss, 'channel')
        RSSFeed.generate_tree(channel, self.dict())
        return etree.tostring(rss, pretty_print=True)
